@@ -1,3 +1,4 @@
+from threading import Lock
 from time import sleep, time
 from math import ceil
 
@@ -35,6 +36,10 @@ class Worker:
         self.on_done = None
 
     def reset_cpu_reservation(self, new_cpu_amount=0):
+        # Note: we rely on the GIL/atomicity of Python bytecode execution
+        # here, since this value will be changed from the handler thread,
+        # but read (via `reserved_cpu_amount()`) in the `wait_for_slot()`
+        # thread.
         self.cpu_reservation_amount = new_cpu_amount
         # Note: we do not call on_done() here, even if the amount would be 0
         # since that would remove a live worker from the list, which would give
@@ -53,17 +58,25 @@ class Worker:
 class WorkerList:
     def __init__(self):
         self.workers = set()
+        self.lock = Lock()
 
     def add_worker(self, worker):
-        worker.on_done = lambda w : self.workers.remove(w)
-        self.workers.add(worker)
+        with self.lock:
+            worker.on_done = lambda w : self.remove_worker(w)
+            self.workers.add(worker)
         return worker
 
+    def remove_worker(self, worker):
+        with self.lock:
+            self.workers.remove(worker)
+
     def count(self):
-        return len(self.workers)
+        with self.lock:
+            return len(self.workers)
 
     def reserved_cpu_amount(self):
-        return sum([w.cpu_reservation_amount for w in self.workers])
+        with self.lock:
+            return sum([w.cpu_reservation_amount for w in self.workers])
 
 
 """Timed measurement on an incrementing counter
